/*
 * AIC support for the X1600.
 *
 * Copyright (C) 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/re/c/dma_space.h>
#include <l4/sys/types.h>
#include <stdint.h>



#ifdef __cplusplus

#include <l4/devices/cpm-x1600.h>
#include <l4/devices/dma-x1600.h>
#include <l4/devices/hw_mmio_register_block.h>

// AIC channel.

class Aic_x1600_channel
{
private:
  l4_addr_t _aic_start;
  Hw::Register_block<32> _regs;
  enum Clock_identifiers _clock_rx, _clock_tx;
  Cpm_x1600_chip *_cpm;
  Dma_x1600_channel *_dma;

  // Buffer management.

  unsigned int _size = 0;
  l4_addr_t _vaddr = 0;
  l4re_dma_space_dma_addr_t _paddr = 0;
  l4_cap_idx_t _mem = L4_INVALID_CAP;

public:
  Aic_x1600_channel(l4_addr_t aic_start, l4_addr_t start,
                    enum Clock_identifiers clock_rx,
                    enum Clock_identifiers clock_tx,
                    Cpm_x1600_chip *cpm,
                    Dma_x1600_channel *dma);

  long get_buffer(uint32_t count, l4_addr_t *addr);
  unsigned int transfer(uint32_t count, uint32_t sample_rate, uint8_t sample_size);

private:
  void disable();
  void enable();

  void config();
  uint32_t encode_input_size(uint8_t sample_size);
  uint32_t encode_output_size(uint8_t sample_size);
  void set_divider(uint32_t divider, uint32_t mask, uint32_t limit, int bit);
  void set_recv_frequency(uint32_t sample_rate);
  void set_trans_frequency(uint32_t sample_rate);
};

// AIC device control.

class Aic_x1600_chip
{
private:
  l4_addr_t _aic_start, _start, _end;
  Cpm_x1600_chip *_cpm;

public:
  Aic_x1600_chip(l4_addr_t aic_start, l4_addr_t start, l4_addr_t end, Cpm_x1600_chip *cpm);

  Aic_x1600_channel *get_channel(uint8_t channel, Dma_x1600_channel *dma);
};

#endif /* __cplusplus */



/* C language interface. */

EXTERN_C_BEGIN

void *x1600_aic_init(l4_addr_t aic_start, l4_addr_t start, l4_addr_t end, void *cpm);

void *x1600_aic_get_channel(void *aic, uint8_t channel, void *dma);

long x1600_aic_get_buffer(void *channel, uint32_t count, l4_addr_t *addr);

unsigned int x1600_aic_transfer(void *channel, uint32_t count, uint32_t sample_rate, uint8_t sample_size);

EXTERN_C_END
